@@ -1,6 +1,6 @@
 /* Fake libiberty.h for Bison.
 
-   Copyright (C) 2002-2004, 2009-2012 Free Software Foundation, Inc.
+   Copyright (C) 2002-2004, 2009-2015 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 
 
 /* Bison depends on libiberty's implementation of bitsets, which
-   requires a `libiberty.h' file.  This file provides the minimum
+   requires a 'libiberty.h' file.  This file provides the minimum
    services.  */
 
 #ifndef BISON_LIBIBERTY_H_
